// WebCustomControlTD.js
// Version: 1.3.1
// By Daniel L. Taylor
// Copyright (C) 2011-2013 Taylor Design. All Rights Reserved.
// http://www.taylor-design.com/
// **********************************************************************

var wcc = new Object;

// Div Accessors **********************************************************************
// Each custom control should exist within a wccControl div, itself within a
// wccContainer div. These divs are used to manage layout and scrolling.
wcc.kContainerDivId = 'wccContainer';
wcc.kControlDivId = 'wccControl';

wcc.GetContainerDiv = function ()
{
	return document.getElementById(wcc.kContainerDivId);
}

wcc.GetControlDiv = function ()
{
	return document.getElementById(wcc.kControlDivId);
}

// Custom Events **********************************************************************
// These variables and functions support custom events which are sent
// back to REALbasic for processing.

// Event parameter constants.
wcc.kSession = 'wccSession';
//wcc.kPage = 'wccPage';
wcc.kControl = 'wccControl';
wcc.kRouteId = 'wccRouteId';
wcc.kEvent = 'wccEvent';
wcc.kPriority = 'wccPriority';
wcc.kBaseUrl = 'wccBaseUrl';
wcc.kEventList = 'wccEventList';
wcc.kQueueType = 'wccQueueType';

// Event ID constants.
wcc.eviMultiEvent = 'wccMultiEvent';

// Event priority constants.
wcc.kNow = 0;
wcc.kSingle = 1;
wcc.kMulti = 2;
wcc.kClear = 3;

// Event lists.
wcc.ClearedEvents = new Array();
wcc.PendingEvents = new Array();

// Event frequency.
wcc.LastEventSent = new Date();
wcc.EventCheckScheduled = false;
wcc.EventFrequency = 50;

wcc.SetEventFrequency = function (intFrequency)
{
	if (wcc.IsNumber(intFrequency) == false)
		intFrequency = 50;
	
	wcc.EventFrequency = intFrequency;
}

wcc.ScheduleEventCheck = function ()
{
	// Preflight
	if (wcc.EventCheckScheduled == true) return;
	wcc.EventCheckScheduled = true;
	
	// Schedule check.
	setTimeout('wcc.DoEventCheck();', wcc.EventFrequency);	
}

wcc.DoEventCheck = function ()
{
	// Clear marker.
	wcc.EventCheckScheduled = false;
	
	// Send events.
	if ((wcc.ClearedEvents.length > 0) || (wcc.PendingEvents.length > 0))
	{
		// Get the events.
		var Events = wcc.ClearedEvents.concat(wcc.PendingEvents);
		wcc.ClearedEvents = new Array();
		wcc.PendingEvents = new Array();
		
		// Send a multievent.
		var objParams = new Object;
		objParams[wcc.kEventList] = Events;
		wcc.RaiseEvent(wcc.kNow, wcc.eviMultiEvent, objParams);
	}
}

// These properties control which events are raised to the RB layer.
// UseEvents is set by the RB layer and includes output from the RB
// UseEvents property and the RB AlwaysUseEvents method.
wcc.UseAllEvents = false;
wcc.UseEvents = '';
wcc.AlwaysUseEvents = 'wccStartup wccShown wccAdjustLayout wccKeepAlive wccMultiEvent wccTest ';

// Mouse button constants.
wcc.ButtonNone = 0;
wcc.ButtonLeft = 1;
wcc.ButtonMiddle = 2;
wcc.ButtonRight = 3;

// Event context.
//wcc.SessionId = '';
//wcc.PageId = '';
//wcc.ControlId = '';
wcc.RouteId = '';
wcc.BaseUrl = '';
wcc.HasStarted = false;

// Called to load the context and startup the control.
wcc.ControlLoaded = function ()
{
	// Get the url parameters.
	var urlVars = wcc.GetUrlVars();
	wcc.SessionId = decodeURIComponent(urlVars[wcc.kSession]);
	//wcc.PageId = decodeURIComponent(urlVars[wcc.kPage]);
	wcc.ControlId = decodeURIComponent(urlVars[wcc.kControl]);
	wcc.RouteId = decodeURIComponent(urlVars[wcc.kRouteId]);
	wcc.BaseUrl = decodeURIComponent(urlVars[wcc.kBaseUrl]);
	wcc.QueueType = urlVars[wcc.kQueueType];

	//Begin startup.
	wcc.SetupScrolling();
	wcc.RaiseEvent(wcc.kNow, "wccStartup");
}

// RaiseEvent
// Use this function to raise events to the RB layer.
// intPriority should be one of the priority constants above.
// strEventId is the event name in RB.
// objParams is a JSON object with the parameters to be passed to the event.
// objParams should NOT be an array, but arrays can be storied as properties
// of the object. Property names that begin with wcc are RESERVED.
//
// Note that this function adheres to UseEvents and event priority.
wcc.RaiseEvent = function (intPriority, strEventId, objParams)
{
	// Raise this event?
	if (wcc.UseAllEvents == false)
		if (wcc.AlwaysUseEvents.indexOf(strEventId) < 0)
			if (wcc.UseEvents.indexOf(strEventId) < 0)
				return;
	
	// Verify context.
	if (wcc.RouteId == '') return;
	
	// Verify objParams.
	if (wcc.RealTypeOf(objParams) != 'Object')
		objParams = new Object();
	
	// Verify priority.
	if (wcc.IsNumber(intPriority) == false)
		intPriority = wcc.kNow;
		
	// Append context variables.
	//objParams[wcc.kSession] = wcc.SessionId;
	//objParams[wcc.kPage] = wcc.PageId;
	//objParams[wcc.kControl] = wcc.ControlId;
	objParams[wcc.kRouteId] = wcc.RouteId;
	objParams[wcc.kEvent] = strEventId;
	objParams[wcc.kPriority] = intPriority;
	
	// Schedule event.
	wcc.ScheduleEvent(objParams);
}

wcc.ScheduleEvent = function (objEvent)
{
	// Preflight
	if (wcc.RealTypeOf(objEvent) != 'Object') return;
	
	// Get the priority.
	var intPriority = objEvent[wcc.kPriority];
	
	// If EventFrequency is 0 then just send the event.
	if (wcc.EventFrequency <= 0)
	{
		wcc.SendEvent(objEvent);
		return;
	}
	
	// Decide what to do with the event based on priority.
	switch(intPriority)
	{
	case wcc.kNow:
		wcc.SendEvent(objEvent);
		return; // We want to return here so that we do not schedule an event check.
	case wcc.kSingle:
		wcc.ReplaceEvent(objEvent);
		break;
	case wcc.kMulti:
		wcc.PendingEvents.push(objEvent);
		break;
	case wcc.kClear:
		wcc.ClearEvents(objEvent);
		break;
	default:
		wcc.SendEvent(objEvent);
	}
	
	// Schedule an event check.
	wcc.ScheduleEventCheck();
}

wcc.ReplaceEvent = function (objEvent)
{
	// Preflight
	if (wcc.RealTypeOf(objEvent) != 'Object') return;
	
	// Setup.
	var strEventId = objEvent[wcc.kEvent];
	var c = wcc.PendingEvents.length;
	var i, objTest;
	
	// Replace first match with same strEventId and priority of kSingle.
	for (i = 0; i < c; i++)
	{
		objTest = wcc.PendingEvents[i];
		if ((objTest[wcc.kEvent] == strEventId) && (objTest[wcc.kPriority] == wcc.kSingle))
		{
			wcc.PendingEvents.splice(i, 1, objEvent);
			return;
		}
	}
	
	// No match was found.
	wcc.PendingEvents.push(objEvent);
}

wcc.ClearEvents = function (objEvent)
{
	wcc.ClearedEvents = wcc.ClearedEvents.concat(wcc.PendingEvents);
	wcc.PendingEvents = new Array();
	wcc.PendingEvents.push(objEvent);
}

wcc.SendEvent = function (objEvent, fncStateChangeHandler)
{
	// Preflight
	if (wcc.RealTypeOf(objEvent) != 'Object')
		return null;
	
	// Create ajaxRequest object.
	var objRequest = wcc.ajaxRequest();
	if (objRequest == false) return null;
	
	if (wcc.RealTypeOf(fncStateChangeHandler) == 'Function')
		objRequest.onreadystatechange = fncStateChangeHandler;
	
	// Send ajaxRequest for our event.
	if (wcc.QueueType == 2)
	{
		//Xojo 2013r1 WebSessionContext bug work around.
		var strUrl = wcc.BaseUrl + '/' + wcc.SessionId + '/error';
		var strParams = 'errorbody=wccEvent&userdetails=' + encodeURIComponent( JSON.stringify(objEvent) );
	}
	else
	{
		var strUrl = wcc.BaseUrl + '/special/wcc/event';
		var strParams = 'wccParameters=' + encodeURIComponent( JSON.stringify(objEvent) );
	}
	
	objRequest.open('POST', strUrl, true);
	objRequest.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	objRequest.setRequestHeader('Cache-Control','no-cache');
	objRequest.send(strParams);

	// Cleanup
	wcc.LastEventSent = new Date();
	return objRequest;
}

// KeepAlive **********************************************************************
// This section exists to address a problem with WCC when running in a WE CGI.
// If a page/session is open but idle for an extended period, the WE
// ExecuteJavaScript function stops working. It has to be called once, then called
// again for it to start working. Naturally this can lead to all kinds
// of problems.
//
// The KeepAlive function will repeatedly raise the KeepAlive event.
// WebCustomControlTD responds to this event by calling this function
// which keeps the js engine alive and sets up the next KeepAlive.
wcc.ScheduleKeepAlive = function (intPeriod)
{
	setTimeout("wcc.RaiseEvent(wcc.kNow, 'wccKeepAlive');", intPeriod);
}

// FastStart
wcc.FastStart = function (objOptions)
{
	// Mark
	wcc.HasStarted = true;
	
	// Preflight
	if (objOptions == null || objOptions == undefined)
		return;
	
	// UseEvents
	wcc.UseAllEvents = objOptions.UseAllEvents;
	wcc.UseEvents = objOptions.UseEvents;
	
	// Event Scheduling
	wcc.SetEventFrequency(objOptions.EventFrequency);
	if (objOptions.KeepAlive == true)
		wcc.ScheduleKeepAlive(30000);
	
	// Scrollbars
	wcc.SetScrollbarVisibleX(objOptions.UseScrollbarX);
	wcc.SetScrollbarVisibleY(objOptions.UseScrollbarY);
	
	// Container Size
	wcc.SetContainerSize(objOptions.ContainerWidth, objOptions.ContainerHeight);
	
	// Control Size
	var strType = wcc.RealTypeOf(objOptions.ControlWidth);
	if (strType == 'Number' || strType == 'String')
		wcc.SetControlSize(objOptions.Width, objOptions.Height);
	
	// Alignment
	wcc.SetAlignHorizontal(objOptions.AlignHorizontal);
	wcc.SetAlignVertical(objOptions.AlignVertical);
}

// Display & Layout **********************************************************************
wcc.kAlignDefault = 0;
wcc.kAlignLeft = 1;
wcc.kAlignTop = 1;
wcc.kAlignCenter = 2;
wcc.kAlignRight = 3;
wcc.kAlignBottom = 3;

wcc.AlignHorizontal = 0;
wcc.AlignVertical = 0;

wcc.SetBackgroundColor = function (color) {
   document.body.style.background = color;
}

wcc.SetContainerSize = function (strWidth, strHeight)
{
	// Get the div.
	var objDiv = wcc.GetContainerDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	// Set the size.
	objDiv.style.width = strWidth;
	objDiv.style.height = strHeight;
}

wcc.SetControlSize = function (strWidth, strHeight)
{
	// Get the div.
	var objDiv = wcc.GetControlDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	// Set the size.
	objDiv.style.width = strWidth;
	objDiv.style.height = strHeight;
}

wcc.SetControlWidth = function (strWidth)
{
	// Get the div.
	var objDiv = wcc.GetControlDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	// Set the size.
	objDiv.style.width = strWidth;
}

wcc.SetControlFontSize = function (strSize)
{
	// Get the div.
	var objDiv = wcc.GetControlDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	// Set the size.
	objDiv.style.fontSize = strSize;
}

wcc.SetControlSizeMin = function (strWidth, strHeight)
{
	// Get the div.
	var objDiv = wcc.GetControlDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	// Set the size.
	objDiv.style.minWidth = strWidth;
	objDiv.style.minHeight = strHeight;
}

wcc.SetControlSizeMax = function (strWidth, strHeight)
{
	// Get the div.
	var objDiv = wcc.GetControlDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	// Set the size.
	objDiv.style.maxWidth = strWidth;
	objDiv.style.maxHeight = strHeight;
}

wcc.SetAlignHorizontal = function (intAlignHorizontal)
{
	// Store value.
	wcc.AlignHorizontal = intAlignHorizontal;
	
	// Get divs.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	var objControl = wcc.GetControlDiv();
	if (objControl === null || objControl === undefined)
		return;
	
	// Compute layout and adjust.
	var x = 0;
	
	switch(wcc.AlignHorizontal)
	{
		case wcc.kAlignDefault:
			x = 0;
			break;
		case wcc.kAlignLeft:
			x = 0;
			break;
		case wcc.kAlignCenter:
			x = (objContainer.offsetWidth - objControl.offsetWidth) / 2;
			break;
		case wcc.kAlignRight:
			x = objContainer.offsetWidth - (objControl.offsetWidth);
			break;
	}

	objControl.style.marginLeft = x + 'px';
}

wcc.SetAlignVertical = function (intAlignVertical)
{
	// Store value.
	wcc.AlignVertical = intAlignVertical;
	
	// Get divs.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	var objControl = wcc.GetControlDiv();
	if (objControl === null || objControl === undefined)
		return;
	
	// Compute layout and adjust.
	var y = 0;
	
	switch(wcc.AlignVertical)
	{
		case wcc.kAlignDefault:
			y = 0;
			break;
		case wcc.kAlignTop:
			y = 0;
			break;
		case wcc.kAlignCenter:
			y = (objContainer.offsetHeight - objControl.offsetHeight) / 2;
			break;
		case wcc.kAlignBottom:
			y = objContainer.offsetHeight - (objControl.offsetHeight);
			break;
	}
	
	objControl.style.marginTop = y + 'px';
}

// Call this function if your client side JavaScript does something
// that requires RB to adjust the layout.
wcc.AdjustLayout = function ()
{
	wcc.RaiseEvent(wcc.kNow, 'wccAdjustLayout');
}

wcc.SetCssLinkById = function (strId, strHref)
{
	var obj = document.getElementById(strId);
	if (obj != null && obj != undefined)
		obj.href = strHref;
}

wcc.LoadHeaderFile = function (strFileName, strFileType)
{
	if (strFileType == "js")
	{ //if filename is a external JavaScript file
		var objFileRef = document.createElement('script');
		objFileRef.setAttribute("type", "text/javascript");
		objFileRef.setAttribute("src", strFileName);
	}
	else if (strFileType=="css")
	{ //if filename is an external CSS file
		var objFileRef = document.createElement("link");
		objFileRef.setAttribute("rel", "stylesheet");
		objFileRef.setAttribute("type", "text/css");
		objFileRef.setAttribute("href", strFileName);
	}
	else
		return;
	
	if (objFileRef != null && objFileRef != undefined)
  		document.getElementsByTagName("head")[0].appendChild(objFileRef);
}

wcc.LoadjQueryUrlStartupVariables = function ()
{
	// Parse the URL for our startup values.
	var urlVars = wcc.GetUrlVars();
	
	// Set font size.
	var strFontSize = urlVars['fontsize'];
	if (typeof(strFontSize) == 'string')
	{
		strFontSize = decodeURIComponent(strFontSize);
		wcc.SetControlFontSize(strFontSize);
	}
	
	// Set theme.
	var strTheme = urlVars['themeurl'];
	if (typeof(strTheme) == 'string')
	{
		strTheme = decodeURIComponent(strTheme);
		wcc.SetCssLinkById('theme', strTheme);
	}
}

wcc.SetStyle = function(strSelector, strProperty, varValue)
{
	var objStyles = document.styleSheets;

	for (var i = 0; i < objStyles.length; i++) 
	{
		try
		{
			var objRules = objStyles[i].cssRules || objStyles[i].rules;
			
			for (var j = 0; j < objRules.length; j++) 
			{
				if (objRules[j].selectorText === strSelector) 
				{
					objRules[j].style[strProperty] = varValue;
				}
			}
		}
		catch (err)
		{
			// Do nothing. We just want to catch cross domain security errors
			// so that we only end up editing local css files.
		}
	}
}

wcc.GetContentSize = function()
{
	// Get the div.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer == null || objContainer == undefined)
		return null;
	
	var objControl = wcc.GetControlDiv();
	if (objControl == null || objControl == undefined)
		return null;
	
	// The height has to come from the control, but the width
	// from the container due to the quirky behavior of the
	// available properties.
	var objSize = new Object;
	objSize.width = objContainer.scrollWidth;
	objSize.height = objControl.scrollHeight;
	return objSize;
}

// Scrolling **********************************************************************
wcc.kScrollbarsAuto = 0;
wcc.kScrollbarsAlways = 1;
wcc.kScrollbarsNever = 2;

wcc.ScrollbarsVisible = 0;
wcc.IgnoreScrollEvent = false;

wcc.SetupScrolling = function()
{
	var objDiv = wcc.GetContainerDiv();
	if (objDiv === null || objDiv === undefined)
		return;
	
	objDiv.onscroll = function(inputDiv)
	{
		if (wcc.IgnoreScrollEvent == true)
		{
			wcc.IgnoreScrollEvent = false;
			return;
		}
		
		wcc.RaiseEvent(wcc.kSingle, "wccContainerScroll", {"x":objDiv.scrollLeft, "y":objDiv.scrollTop});
	}
}

wcc.SetScrollbarsVisible = function (intX, intY)
{
	wcc.SetScrollbarsVisibleX(intX);
	wcc.SetScrollbarsVisibleY(intY);
}

wcc.SetScrollbarVisibleX = function (intVisible)
{
	// Store value.
	wcc.ScrollbarsVisibleX = intVisible;
	
	// Update div.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	switch (wcc.ScrollbarsVisibleX)
	{
		case wcc.kScrollbarsAuto:
			objContainer.style.overflowX = "auto";
			break;
		case wcc.kScrollbarsAlways:
			objContainer.style.overflowX = "scroll";
			break;
		case wcc.kScrollbarsNever:
			objContainer.style.overflowX = "hidden";
			break
	}
}

wcc.SetScrollbarVisibleY = function (intVisible)
{
	// Store value.
	wcc.ScrollbarsVisibleY = intVisible;
	
	// Update div.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	switch (wcc.ScrollbarsVisibleY)
	{
		case wcc.kScrollbarsAuto:
			objContainer.style.overflowY = "auto";
			break;
		case wcc.kScrollbarsAlways:
			objContainer.style.overflowY = "scroll";
			break;
		case wcc.kScrollbarsNever:
			objContainer.style.overflowY = "hidden";
			break
	}
}

wcc.ScrollContainerLeft = function (intValue, bolIgnoreEvent)
{
	// Preflight
	if (isNaN(intValue) == true)
		return;
		
	if (bolIgnoreEvent != true && bolIgnoreEvent != false)
		bolIgnoreEvent = false;
	
	// Get the container.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	// Scroll the container.
	wcc.IgnoreScrollEvent = bolIgnoreEvent;	
	objContainer.scrollLeft = intValue;
}

wcc.ScrollContainerLeftAsync = function (intValue, intDelay)
{
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	setTimeout(function(){objContainer.scrollLeft = intValue}, intDelay);
}

wcc.ScrollContainerTop = function (intValue, bolIgnoreEvent)
{
	// Preflight
	if (isNaN(intValue) == true)
		return;
		
	if (bolIgnoreEvent != true && bolIgnoreEvent != false)
		bolIgnoreEvent = false;
	
	// Get the container.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	// Scroll the container.
	wcc.IgnoreScrollEvent = bolIgnoreEvent;	
	objContainer.scrollTop = intValue;
}


wcc.ScrollContainerTopAsync = function (intValue, intDelay)
{
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	setTimeout(function(){objContainer.scrollTop = intValue}, intDelay);
}

// AJAX Support **********************************************************************
wcc.ajaxRequest = function ()
{
	//Test for support for ActiveXObject in IE first (as XMLHttpRequest in IE7 is broken).
	var activexmodes= ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP'];
	if (window.ActiveXObject)
	{
		for (var i = 0; i < activexmodes.length; i++)
		{
			try
			{
				return new ActiveXObject(activexmodes[i]);
			}
			catch(e)
			{
				//suppress error
			}
		}
	}
	else if (window.XMLHttpRequest) // if Mozilla, Safari etc
		return new XMLHttpRequest();
	else
		return false;
}

// Key Events
wcc.GetKeyEvent = function(objEvent)
{
	var objKeyEvent = new Object;
	
	objKeyEvent.altKey = objEvent.altKey;
	objKeyEvent.charCode = objEvent.charCode;
	objKeyEvent.ctrlKey = objEvent.ctrlKey;
	objKeyEvent.keyCode = objEvent.keyCode;
	objKeyEvent.metaKey = objEvent.metaKey;
	objKeyEvent.shiftKey = objEvent.shiftKey;
	
	return objKeyEvent;
}

// Testing
wcc.ShowObjectValues = function (obj)
{
	for (myKey in obj)
	{
		alert ("data["+myKey +"] = " + obj[myKey]);
	}
}

// Utilities **********************************************************************
// Functions below this line were modified from public domain sources.
wcc.CreateUuid = function () {
    var s = [];
    var hexDigits = "0123456789ABCDEF";
    for (var i = 0; i < 64; i++) {
        s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
    }
    s[12] = "4";  // bits 12-15 of the time_hi_and_version field to 0010
    s[16] = hexDigits.substr((s[16] & 0x3) | 0x8, 1);  // bits 6-7 of the clock_seq_hi_and_reserved to 01

    var uuid = s.join("");
    return uuid;
}

wcc.Pause = function (intMilliseconds)
{
	// This isn't really a good idea in js as it blocks the web browser.
	// But I have found short pauses necessary in some locations.
	var objDate = new Date();
	var objCurDate = null;
	do { objCurDate = new Date(); }
	while(objCurDate - objDate < intMilliseconds);
}

wcc.GetUrlVars = function ()
{
	// Read a page's GET URL variables and return them as an associative array.
    var vars = [], hash;
    var hashes = window.location.href.slice(window.location.href.indexOf('?') + 1).split('&');
    for(var i = 0; i < hashes.length; i++)
    {
        hash = hashes[i].split('=');
        vars.push(hash[0]);
        vars[hash[0]] = hash[1];
    }
    return vars;
}

wcc.IsNumber = function (varValue) 
{
  return (typeof varValue == 'string' || typeof varValue == 'number') && !isNaN(varValue - 0) && varValue !== '';
};

wcc.RealTypeOf = function(obj) 
{
    return Object.prototype.toString.call(obj).slice(8, -1);
};
